# -*- coding: utf-8 -*-

# Copyright (c) 2015 by intelligenia <info@intelligenia.es>
#
# The MIT License (MIT)
#
# Copyright (c) 2016 intelligenia soluciones informáticas

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from setuptools import setup, Command, find_packages
from shutil import rmtree
import os
import sys
import io

# Package meta-data.
NAME = "django-virtual-pos"
DESCRIPTION = "django-virtual-pos is a module that abstracts the flow of paying in several online payment platforms."
URL = 'https://github.com/intelligenia/django-virtual-pos'
EMAIL = 'mario@intelligenia.com'
AUTHOR = 'intelligenia'
REQUIRES_PYTHON = '>=2.7.0'
VERSION = None
KEYWORDS = ["virtual", "point-of-sale", "puchases", "online", "payments"]

# Directory with the package
PACKAGE = "djangovirtualpos"

# What packages are required for this module to be executed?
REQUIRED = [
    "django",
    "beautifulsoup4",
    "lxml",
    "pycrypto",
    "pytz",
    "requests"
]

# The rest you shouldn't have to touch too much :)
# ------------------------------------------------
# Except, perhaps the License and Trove Classifiers!
# If you do change the License, remember to change the Trove Classifier for that!

here = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long-description.
# Note: this will only work if 'README.rst' is present in your MANIFEST.in file!
with io.open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = '\n' + f.read()

# Load the package's __version__.py module as a dictionary.
about = {}
if not VERSION:
    with open(os.path.join(here, PACKAGE, '__version__.py')) as f:
        exec (f.read(), about)
else:
    about['__version__'] = VERSION


class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPi via Twine…')
        os.system('twine upload dist/*')

        self.status('Pushing git tags…')
        os.system('git tag v{0}'.format(about['__version__']))
        os.system('git push --tags')

        sys.exit()


setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=long_description,
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(exclude=('tests',)),
    # If your package is a single module, use this instead of 'packages':
    # py_modules=['mypackage'],

    # entry_points={
    #     'console_scripts': ['mycli=mymodule:cli'],
    # },
    install_requires=REQUIRED,
    include_package_data=True,
    license="MIT",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django',
        'License :: OSI Approved :: MIT License',
    ],
    keywords=KEYWORDS,
    cmdclass={
        'upload': UploadCommand,
    },
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangovirtualpos', '0003_auto_20170310_1829'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='virtualpointofsale',
            options={'ordering': ['name'], 'verbose_name': 'virtual point of sale', 'verbose_name_plural': 'virtual points of sale', 'permissions': (('view_virtualpointofsale', 'Can view Virtual Points of Sale'),)},
        ),
    ]



# coding=utf-8

from django.contrib import admin
from djangovirtualpos.models import VirtualPointOfSale, VPOSRefundOperation, VPOSCeca, VPOSRedsys, VPOSSantanderElavon, VPOSPaypal, VPOSBitpay

admin.site.register(VirtualPointOfSale)
admin.site.register(VPOSRefundOperation)
admin.site.register(VPOSCeca)
admin.site.register(VPOSRedsys)
admin.site.register(VPOSPaypal)
admin.site.register(VPOSSantanderElavon)
admin.site.register(VPOSBitpay)





# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.core.exceptions import ObjectDoesNotExist
from django.db import transaction
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import render, get_object_or_404
from django.urls import reverse
from django.views.decorators.csrf import csrf_exempt
from djangovirtualpos.models import VirtualPointOfSale, VPOSCantCharge


from django.http import JsonResponse


def set_payment_attributes(request, sale_model, sale_ok_url, sale_nok_url, reference_number=False):
    """
    Set payment attributes for the form that makes the first call to the VPOS.
    :param request: HttpRequest.
    :param sale_model: Sale model. Must have "status" and "operation_number" attributes and "online_confirm" method.
    :param sale_ok_url: Name of the URL used to redirect client when sale is successful.
    :param sale_nok_url: Name of the URL used to redirect client when sale is not successful.
    :return: HttpResponse.
    """
    if request.method == 'GET':
        return JsonResponse({"message":u"Method not valid."})

    # Getting the VPOS and the Sale
    try:
        # Getting the VirtualPointOfSale object
        virtual_point_of_sale = VirtualPointOfSale.get(id=request.POST["vpos_id"], is_erased=False)
        # Getting Sale object
        payment_code = request.POST["payment_code"]
        sale = sale_model.objects.get(code=payment_code, status="pending")
        sale.virtual_point_of_sale = virtual_point_of_sale
        sale.save()

    except ObjectDoesNotExist as e:
        return JsonResponse({"message":u"La orden de pago no ha sido previamente creada."}, status=404)

    except VirtualPointOfSale.DoesNotExist:
        return JsonResponse({"message": u"VirtualPOS does NOT exist"}, status=404)

    virtual_point_of_sale.configurePayment(
        # Payment amount
        amount=sale.amount,
        # Payment description
        description=sale.description,
        # Sale code
        sale_code=sale.code,
        # Return URLs
        url_ok=request.build_absolute_uri(reverse(sale_ok_url, kwargs={"sale_code": sale.code})),
        url_nok=request.build_absolute_uri(reverse(sale_nok_url, kwargs={"sale_code": sale.code})),
    )

    # Operation number assignment. This operation number depends on the
    # Virtual VPOS selected, it can be letters and numbers or numbers
    # or even match with a specific pattern depending on the
    # Virtual VPOS selected, remember.
    try:
        # Operation number generation and assignement
        operation_number = virtual_point_of_sale.setupPayment()
        # Update operation number of sale
        sale.operation_number = operation_number
        sale_model.objects.filter(id=sale.id).update(operation_number=operation_number)
    except Exception as e:
        return JsonResponse({"message": u"Error generating operation number {0}".format(e)}, status=500)

    # Payment form data
    if hasattr(reference_number, "lower") and reference_number.lower() == "request":
        form_data = virtual_point_of_sale.getPaymentFormData(reference_number="request")
    elif reference_number:
        form_data = virtual_point_of_sale.getPaymentFormData(reference_number=reference_number)
    else:
        form_data = virtual_point_of_sale.getPaymentFormData(reference_number=False)

    # Debug message
    form_data["message"] = "Payment {0} updated. Returning payment attributes.".format(payment_code)

    # Return JSON response
    return JsonResponse(form_data)


# Confirm sale
@csrf_exempt
def confirm_payment(request, virtualpos_type, sale_model):
    """
    This view will be called by the bank.
    """
    # Checking if the Point of Sale exists
    virtual_pos = VirtualPointOfSale.receiveConfirmation(request, virtualpos_type=virtualpos_type)

    if not virtual_pos:
        # The VPOS does not exist, inform the bank with a cancel
        # response if needed
        return VirtualPointOfSale.staticResponseNok(virtualpos_type)

    # Verify if bank confirmation is indeed from the bank
    verified = virtual_pos.verifyConfirmation()
    operation_number = virtual_pos.operation.operation_number

    with transaction.atomic():
        try:
            # Getting your payment object from operation number
            payment = sale_model.objects.get(operation_number=operation_number, status="pending")
        except ObjectDoesNotExist:
            return virtual_pos.responseNok("not_exists")

        if verified:
            # Charge the money and answer the bank confirmation
            try:
                response = virtual_pos.charge()
                # Implement the online_confirm method in your payment
                # this method will mark this payment as paid and will
                # store the payment date and time.
                payment.virtual_pos = virtual_pos
                payment.online_confirm()
            except VPOSCantCharge as e:
                return virtual_pos.responseNok(extended_status=e)
            except Exception as e:
                return virtual_pos.responseNok("cant_charge")

        else:
            # Payment could not be verified
            # signature is not right
            response = virtual_pos.responseNok("verification_error")

        return response


# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2018-04-03 09:53
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangovirtualpos', '0012_auto_20180209_1222'),
    ]

    operations = [
        migrations.AddField(
            model_name='vposredsys',
            name='enable_preauth_policy',
            field=models.BooleanField(default=False, verbose_name='Habilitar pol\xedtica de preautorizaci\xf3n'),
        ),
    ]
